'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (_ref) {
	var t = _ref.types;

	function convertAttributeValue(node) {
		if (t.isJSXExpressionContainer(node)) {
			return node.expression;
		} else {
			return node;
		}
	}

	function fixupPropertyName(propertyName) {
		var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

		var nameValue = propertyName.name;
		if (opts.jsxCompliant) {
			if (t.isIdentifier(propertyName) || t.isJSXIdentifier(propertyName)) {
				nameValue = (0, _eslintUnknownProperty.getStandardName)(nameValue);
			}
		} else {
			// Old/mithril behaviour
			if ((t.isIdentifier(propertyName) || t.isJSXIdentifier(propertyName)) && nameValue === 'class') {
				nameValue = 'className';
			}
		}
		return t.isValidIdentifier(nameValue) ? t.identifier(nameValue) : t.stringLiteral(nameValue);
	}

	function convertAttribute(node) {
		var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

		var propertyName = fixupPropertyName(node.name, opts);
		var value = convertAttributeValue(node.value || t.booleanLiteral(true));

		if (t.isStringLiteral(value)) {
			value.value = value.value.replace(/\n\s+/g, " ");
		}

		return t.inherits(t.objectProperty(propertyName, value), node);
	}

	function _stringLiteralTrimmer(lastNonEmptyLine, lineCount, line, i) {
		var isFirstLine = i === 0;
		var isLastLine = i === lineCount - 1;
		var isLastNonEmptyLine = i === lastNonEmptyLine;

		// replace rendered whitespace tabs with spaces
		var trimmedLine = line.replace(/\t/g, " ");

		// trim leading whitespace
		if (!isFirstLine) {
			trimmedLine = trimmedLine.replace(/^[ ]+/, "");
		}

		// trim trailing whitespace
		if (!isLastLine) {
			trimmedLine = trimmedLine.replace(/[ ]+$/, "");
		}

		if (trimmedLine.length > 0) {
			if (!isLastNonEmptyLine) {
				trimmedLine += " ";
			}

			return trimmedLine;
		}
		return '';
	}

	function cleanStringLiteral(value) {
		var lines = value.split(/\r\n|\n|\r/);

		var lastNonEmptyLine = 0;

		for (var i = lines.length - 1; i > 0; i--) {
			if (lines[i].match(/[^ \t]/)) {
				lastNonEmptyLine = i;
				break;
			}
		}

		var str = lines.map(_stringLiteralTrimmer.bind(null, lastNonEmptyLine, lines.length)).filter(function (line) {
			return line.length > 0;
		}).join('');

		if (str.length > 0) {
			return t.stringLiteral(str);
		}
	}

	function buildChildren(node) {
		return node.children.map(convertAttributeValue).filter(function (child) {
			return !t.isJSXEmptyExpression(child);
		}).map(function (child) {
			if (t.isStringLiteral(child) || t.isJSXText(child)) {
				return cleanStringLiteral(child.value);
			}
			return child;
		}).filter(function (child) {
			return !!child;
		});
	}

	function flatten(args) {
		var flattened = [];
		var last = null;

		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {
			for (var _iterator = args[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				var arg = _step.value;

				if (t.isStringLiteral(arg) && t.isStringLiteral(last)) {
					last.value += arg.value;
				} else {
					last = arg;
					flattened.push(arg);
				}
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator.return) {
					_iterator.return();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}

		return flattened;
	}

	return {
		inherits: require('babel-plugin-syntax-jsx'),

		visitor: {
			JSXElement: function JSXElement(path, _ref2) {
				var file = _ref2.file;
				var opts = _ref2.opts;
				var node = path.node;
				var parent = path.parent;
				var scope = path.scope;

				var open = node.openingElement;
				var obj = t.objectExpression([]);
				var tag = open.name;
				var attrObjs = [];
				var props = [];
				var isComp = false;

				function pushElemProp(key, value) {
					obj.properties.push(t.objectProperty(t.identifier(key), value));
				}

				function pushProps() {
					if (!props.length) {
						return;
					}
					attrObjs.push(t.objectExpression(props));
					props = [];
				}

				if (t.isJSXIdentifier(tag)) {
					tag = t.stringLiteral(tag.name);
					if (opts.warnDeprecatedHtml && deprecatedRx.test(tag.value)) {
						console.log('Using a deprecated html tag: \'' + tag.value + '\'');
					}
					if (tag.value !== tag.value.toLowerCase()) {
						isComp = true;
					}
				}

				pushElemProp('tag', tag);

				if (node.children.length) {
					pushElemProp('children', t.arrayExpression(flatten(buildChildren(node))));
				}

				var _iteratorNormalCompletion2 = true;
				var _didIteratorError2 = false;
				var _iteratorError2 = undefined;

				try {
					for (var _iterator2 = open.attributes[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
						var attr = _step2.value;

						if (t.isJSXSpreadAttribute(attr)) {
							pushProps();
							attrObjs.push(attr.argument);
						} else {
							props.push(convertAttribute(attr, opts));
						}
					}
				} catch (err) {
					_didIteratorError2 = true;
					_iteratorError2 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion2 && _iterator2.return) {
							_iterator2.return();
						}
					} finally {
						if (_didIteratorError2) {
							throw _iteratorError2;
						}
					}
				}

				pushProps();

				if (attrObjs.length === 1) {
					pushElemProp('attrs', attrObjs[0]);
				} else if (attrObjs.length > 1) {
					if (!t.isObjectExpression(attrObjs[0])) {
						attrObjs.unshift(t.objectExpression([]));
					}

					pushElemProp('attrs', t.callExpression(file.addHelper('extends'), attrObjs));
				}

				if (isComp) {
					var properties = obj.properties;
					var attrs = t.objectExpression([]);
					var children = t.arrayExpression([]);

					var _iteratorNormalCompletion3 = true;
					var _didIteratorError3 = false;
					var _iteratorError3 = undefined;

					try {
						for (var _iterator3 = properties[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
							var child = _step3.value;

							if (child.key.name === 'attrs') {
								attrs = child.value;
							} else if (child.key.name === 'children') {
								children = child.value;
							}
						}
					} catch (err) {
						_didIteratorError3 = true;
						_iteratorError3 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion3 && _iterator3.return) {
								_iterator3.return();
							}
						} finally {
							if (_didIteratorError3) {
								throw _iteratorError3;
							}
						}
					}

					path.replaceWith(t.callExpression(t.memberExpression(t.identifier('m'), t.identifier('component')), [t.identifier(tag.value), attrs, children]), node);
				} else {
					path.replaceWith(obj, node);
				}
			}
		}
	};
};

var _eslintUnknownProperty = require('./eslint-unknown-property');

var deprecatedHTML = '^(acronym|applet|basefont|big|blink|center|dir|frame|frameset|isindex|listings|noembed|plaintext|spacer|strike|tt|xmp)$';

var deprecatedRx = new RegExp(deprecatedHTML);

function D(v) {
	console.log(JSON.stringify(v, null, '\t'));
}

;